@@ -5,8 +5,8 @@
 * Botan is released under the Simplified BSD License (see license.txt)
 */
 
-#ifndef BOTAN_MD5_H__
-#define BOTAN_MD5_H__
+#ifndef BOTAN_MD5_H_
+#define BOTAN_MD5_H_
 
 #include <botan/mdx_hash.h>
 
@@ -15,22 +15,23 @@ namespace Botan {
 /**
 * MD5
 */
-class BOTAN_DLL MD5 final : public MDx_HashFunction
+class BOTAN_PUBLIC_API(2,0) MD5 final : public MDx_HashFunction
    {
    public:
       std::string name() const override { return "MD5"; }
       size_t output_length() const override { return 16; }
       HashFunction* clone() const override { return new MD5; }
+      std::unique_ptr<HashFunction> copy_state() const override;
 
       void clear() override;
 
       MD5() : MDx_HashFunction(64, false, true), m_M(16), m_digest(4)
          { clear(); }
-   protected:
+
+   private:
       void compress_n(const uint8_t[], size_t blocks) override;
       void copy_out(uint8_t[]) override;
 
-   private:
       /**
       * The message buffer
       */
